Maybe this which includes your default-path change in a slightly
different form and renames the functions to cmd_default_path and
cmdq_default_path.

diff --git a/cmd-new-session.c b/cmd-new-session.c
index 4eebe63..80829f0 100644
--- a/cmd-new-session.c
+++ b/cmd-new-session.c
@@ -35,9 +35,10 @@ enum cmd_retval       cmd_new_session_exec(struct cmd *, 
struct cmd_q *);
 
 const struct cmd_entry cmd_new_session_entry = {
        "new-session", "new",
-       "AdDF:n:Ps:t:x:y:", 0, 1,
-       "[-AdDP] [-F format] [-n window-name] [-s session-name] "
-       CMD_TARGET_SESSION_USAGE " [-x width] [-y height] [command]",
+       "Ac:dDF:n:Ps:t:x:y:", 0, 1,
+       "[-AdDP] [-c start-directory] [-F format] [-n window-name] "
+       "[-s session-name] " CMD_TARGET_SESSION_USAGE " [-x width] [-y height] "
+       "[command]",
        CMD_STARTSERVER|CMD_CANTNEST|CMD_SENDENVIRON,
        NULL,
        cmd_new_session_check,
@@ -62,8 +63,8 @@ cmd_new_session_exec(struct cmd *self, struct cmd_q *cmdq)
        struct environ           env;
        struct termios           tio, *tiop;
        struct passwd           *pw;
-       const char              *newname, *target, *update, *cwd, *errstr;
-       const char              *template;
+       const char              *newname, *target, *update, *base, *cwd;
+       const char              *errstr, *template;
        char                    *cmd, *cause, *cp;
        int                      detached, idx;
        u_int                    sx, sy;
@@ -131,14 +132,19 @@ cmd_new_session_exec(struct cmd *self, struct cmd_q *cmdq)
 
        /* Get the new session working directory. */
        if (c != NULL && c->cwd != NULL)
-               cwd = c->cwd;
+               base = c->cwd;
        else {
                pw = getpwuid(getuid());
                if (pw->pw_dir != NULL && *pw->pw_dir != '\0')
-                       cwd = pw->pw_dir;
+                       base = pw->pw_dir;
                else
-                       cwd = "/";
+                       base = "/";
        }
+       if (args_has(args, 'c'))
+               cwd = args_get(args, 'c');
+       else
+               cwd = options_get_string(&global_s_options, "default-path");
+       cwd = cmd_default_path(base, base, cwd);
 
        /* Find new session size. */
        if (c != NULL) {
diff --git a/cmd-new-window.c b/cmd-new-window.c
index cfc0b8b..976b62d 100644
--- a/cmd-new-window.c
+++ b/cmd-new-window.c
@@ -103,7 +103,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_q *cmdq)
                cmd = options_get_string(&s->options, "default-command");
        else
                cmd = args->argv[0];
-       cwd = cmd_get_default_path(cmdq, args_get(args, 'c'));
+       cwd = cmdq_default_path(cmdq, args_get(args, 'c'));
 
        if (idx == -1)
                idx = -1 - options_get_number(&s->options, "base-index");
diff --git a/cmd-queue.c b/cmd-queue.c
index 38a88d2..bbab1db 100644
--- a/cmd-queue.c
+++ b/cmd-queue.c
@@ -283,3 +283,27 @@ cmdq_flush(struct cmd_q *cmdq)
        }
        cmdq->item = NULL;
 }
+
+/* Get default path using command queue. */
+const char *
+cmdq_default_path(struct cmd_q *cmdq, const char *cwd)
+{
+       struct client   *c = cmdq->client;
+       struct session  *s;
+       const char      *this;
+
+       if ((s = cmd_current_session(cmdq, 0)) == NULL)
+               return (NULL);
+
+       if (cwd == NULL)
+               cwd = options_get_string(&s->options, "default-path");
+
+       if (c != NULL && c->session == NULL && c->cwd != NULL)
+               this = c->cwd;
+       else if (s->curw != NULL)
+               this = osdep_get_cwd(s->curw->window->active->fd);
+       else
+               this = NULL;
+
+       return (cmd_default_path(s->cwd, this, cwd));
+}
diff --git a/cmd-split-window.c b/cmd-split-window.c
index 601dcb1..1c7a1ea 100644
--- a/cmd-split-window.c
+++ b/cmd-split-window.c
@@ -83,7 +83,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_q *cmdq)
                cmd = options_get_string(&s->options, "default-command");
        else
                cmd = args->argv[0];
-       cwd = cmd_get_default_path(cmdq, args_get(args, 'c'));
+       cwd = cmdq_default_path(cmdq, args_get(args, 'c'));
 
        type = LAYOUT_TOPBOTTOM;
        if (args_has(args, 'h'))
diff --git a/tmux.1 b/tmux.1
index c93f4d9..a33f9ad 100644
--- a/tmux.1
+++ b/tmux.1
@@ -673,6 +673,7 @@ Lock all clients attached to
 .Ar target-session .
 .It Xo Ic new-session
 .Op Fl AdDP
+.Op Fl c Ar start-directory
 .Op Fl F Ar format
 .Op Fl n Ar window-name
 .Op Fl s Ar session-name
diff --git a/tmux.h b/tmux.h
index 6ab9861..44d30a8 100644
--- a/tmux.h
+++ b/tmux.h
@@ -1767,7 +1767,7 @@ int                cmd_find_index(struct cmd_q *, const 
char *,
 struct winlink *cmd_find_pane(struct cmd_q *, const char *, struct session **,
                     struct window_pane **);
 char           *cmd_template_replace(const char *, const char *, int);
-const char             *cmd_get_default_path(struct cmd_q *, const char *);
+const char             *cmd_default_path(const char *, const char *, const 
char *);
 extern const struct cmd_entry *cmd_table[];
 extern const struct cmd_entry cmd_attach_session_entry;
 extern const struct cmd_entry cmd_bind_key_entry;
@@ -1876,6 +1876,7 @@ void               cmdq_run(struct cmd_q *, struct 
cmd_list *);
 void            cmdq_append(struct cmd_q *, struct cmd_list *);
 int             cmdq_continue(struct cmd_q *);
 void            cmdq_flush(struct cmd_q *);
+const char             *cmdq_default_path(struct cmd_q *, const char *);
 
 /* cmd-string.c */
 int    cmd_string_parse(const char *, struct cmd_list **, const char *,


On Wed, Aug 21, 2013 at 04:26:41PM -0500, J Raynor wrote:
> There are some conditions I don't know how to generate, so I couldn't
> test those.  But I tried testing what I could, and it appears that
> this preserves the behavior of neww and split-window, and that the
> prefixes for "new -c" match the -c behavior for neww and split-window.
> 
> Does cmd_default_path1 need to be a separate function?  It's short and
> it's only called by cmd_default_path.
> 
> In cmd_default_path0, can anything actually reach the "return (base);"
> statement that's just above the "find_home:" label?
> 
> Cmd-new-session doesn't honor "default-path" when setting the cwd of
> the first pane created in the new session.  I realize this is existing
> behavior and not specific to your patch.  But should it honor that
> setting?
> 
> If so, then take a look at the attached patch.  It's basically your
> patch with a little extra in cmd-new-session.c to get new-session to
> set the first pane's cwd to default-path.



------------------------------------------------------------------------------
Introducing Performance Central, a new site from SourceForge and 
AppDynamics. Performance Central is your source for news, insights, 
analysis and resources for efficient Application Performance Management. 
Visit us today!
http://pubads.g.doubleclick.net/gampad/clk?id=48897511&iu=/4140/ostg.clktrk
_______________________________________________
tmux-users mailing list
tmux-users@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/tmux-users

Reply via email to
diff --git a/cmd.c b/cmd.c
index 282fb11..4525a26 100644
--- a/cmd.c
+++ b/cmd.c
@@ -1283,29 +1283,20 @@ cmd_template_replace(const char *template, const char 
*s, int idx)
  * Return the default path for a new pane, using the given path or the
  * default-path option if it is NULL. Several special values are accepted: the
  * empty string or relative path for the current pane's working directory, ~
  * for the user's home, - for the session working directory, . for the tmux
  * server's working directory. The default on failure is the session's working
  * directory.
  */
 const char *
-cmd_get_default_path(struct cmd_q *cmdq, const char *cwd)
+cmd_default_path(const char *base, const char *this, const char *cwd)
 {
-       struct client           *c = cmdq->client;
-       struct session          *s;
-       struct environ_entry    *envent;
        const char              *root;
+       struct environ_entry    *envent;
        char                     tmp[MAXPATHLEN];
        struct passwd           *pw;
        int                      n;
        size_t                   skip;
        static char              path[MAXPATHLEN];
 
-       if ((s = cmd_current_session(cmdq, 0)) == NULL)
-               return (NULL);
-
-       if (cwd == NULL)
-               cwd = options_get_string(&s->options, "default-path");
-
        skip = 1;
        if (strcmp(cwd, "$HOME") == 0 || strncmp(cwd, "$HOME/", 6) == 0) {
                /* User's home directory - $HOME. */
@@ -1315,8 +1306,8 @@ cmd_get_default_path(struct cmd_q *cmdq, const char *cwd)
                /* User's home directory - ~. */
                goto find_home;
        } else if (cwd[0] == '-' && (cwd[1] == '\0' || cwd[1] == '/')) {
-               /* Session working directory. */
-               root = s->cwd;
+               /* Base working directory. */
+               root = base;
                goto complete_path;
        } else if (cwd[0] == '.' && (cwd[1] == '\0' || cwd[1] == '/')) {
                /* Server working directory. */
@@ -1324,24 +1315,21 @@ cmd_get_default_path(struct cmd_q *cmdq, const char 
*cwd)
                        root = tmp;
                        goto complete_path;
                }
-               return (s->cwd);
+               return ("/");
        } else if (*cwd == '/') {
                /* Absolute path. */
                return (cwd);
        } else {
                /* Empty or relative path. */
-               if (c != NULL && c->session == NULL && c->cwd != NULL)
-                       root = c->cwd;
-               else if (s->curw != NULL)
-                       root = osdep_get_cwd(s->curw->window->active->fd);
+               if (this != NULL)
+                       root = this;
                else
-                       return (s->cwd);
+                       return (base);
                skip = 0;
-               if (root != NULL)
-                       goto complete_path;
+               goto complete_path;
        }
 
-       return (s->cwd);
+       return (base);
 
 find_home:
        envent = environ_find(&global_environ, "HOME");
@@ -1350,7 +1338,7 @@ find_home:
        else if ((pw = getpwuid(getuid())) != NULL)
                root = pw->pw_dir;
        else
-               return (s->cwd);
+               return (base);
 
 complete_path:
        if (root[skip] == '\0') {
@@ -1360,5 +1348,5 @@ complete_path:
        n = snprintf(path, sizeof path, "%s/%s", root, cwd + skip);
        if (n > 0 && (size_t)n < sizeof path)
                return (path);
-       return (s->cwd);
+       return (base);
 }
@@ -1283,29 +1283,20 @@ cmd_template_replace(const char *template, const char 
  * default-path option if it is NULL. Several special values are accepted: the
  * empty string or relative path for the current pane's working directory, ~
- * for the user's home, - for the session working directory, . for the tmux
- * server's working directory. The default on failure is the session's working
- * directory.
+ * for the user's home, - for the base working directory, . for the server
+ * working directory.
  */
 const char *